﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using DFWV.WorldClasses;

namespace DFWV
{
    internal partial class FilterForm : Form
    {
        readonly Dictionary<string, Type> Fields = new Dictionary<string, Type>();
        readonly Dictionary<string, IEnumerable<string>> Options = new Dictionary<string, IEnumerable<string>>();

        readonly string[] Ops = { "=", "!=", ">", "<", ">=", "<=", "Contains", "Doesn't Contain", "true", "false" };

        readonly string[] intOps = { "=", "!=", ">", "<", ">=", "<="};
        readonly string[] stringOps = { "=", "!=", ">", "<", ">=", "<=", "Contains", "Doesn't Contain"};
        readonly string[] boolOps = { "true", "false" };
        // string[] listOps = { "=", "!=" };

        public Filter outFilter;


        public FilterForm(World world, Type filterType)
        {
            InitializeComponent();
            cmbWhereOperation.Items.AddRange(Ops.ToArray<object>());
            cmbOrderOperation.Items.AddRange(Ops.ToArray<object>());

            Fields = world.Filters.Fields[filterType];
            cmbWhereField.Items.AddRange(Fields.Keys.ToArray<object>());
            cmbOrderField.Items.AddRange(Fields.Keys.ToArray<object>());

            if (world.Filters.Options.ContainsKey(filterType))
            {
                Options = world.Filters.Options[filterType];

                cmbWhereField.Items.AddRange(Options.Keys.ToArray<object>());
                cmbOrderField.Items.AddRange(Options.Keys.ToArray<object>());
            }
            else
            {
                Options = null;
            }

            lstWhere.Items.AddRange(world.Filters[filterType].Where.ToArray<object>());
            lstOrder.Items.AddRange(world.Filters[filterType].OrderBy.ToArray<object>());

            chkTake.Checked = world.Filters[filterType].Take != -1;
            txtTake.Text = world.Filters[filterType].Take != -1 ? world.Filters[filterType].Take.ToString() : "";
        }


        private void chkTake_CheckedChanged(object sender, EventArgs e)
        {
            txtTake.Enabled = chkTake.Checked;
        }

        private void cmbWhereField_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selected = ((ComboBox)sender).SelectedItem.ToString();
            cmbWhereOperation.Items.Clear();
            if (Fields.ContainsKey(selected))
            {
                if (Fields[selected] == typeof(bool))
                {
                    cmbWhereOperation.Items.AddRange(boolOps.ToArray<object>());
                    cmbWhereOperation.SelectedIndex = 0;
                    txtWhereData.Visible = false;
                }
                else if (Fields[selected] == typeof(int))
                {
                    cmbWhereOperation.Items.AddRange(intOps.ToArray<object>());
                    txtWhereData.Visible = true;
                }
                else if (Fields[selected] == typeof(string))
                {
                    cmbWhereOperation.Items.AddRange(stringOps.ToArray<object>());
                    txtWhereData.Visible = true;
                }
                
            }
            else
            {
                var addRange = Options[selected].ToList();
                addRange.Sort();                
                cmbWhereOperation.Items.AddRange(addRange.ToArray<object>());
                txtWhereData.Visible = false;
            }
            txtWhereData.Text = "";
        }

        private void WhereAdd_Click(object sender, EventArgs e)
        {
            if (cmbWhereOperation.SelectedIndex == -1 ||
                cmbWhereField.SelectedIndex == -1)
            {
                WhereAdd.BackColor = Color.Red;
                return;
            }
            var selected = cmbWhereField.SelectedItem.ToString();
            var thisOp = cmbWhereOperation.SelectedItem.ToString();
            var DataInt = 0;
            var Data = "";

            if (Fields.ContainsKey(selected))
            {
                if (Fields[selected] == typeof(int))
                {
                    if (txtWhereData.Text == "")
                    {
                        WhereAdd.BackColor = Color.Red;
                        return;
                    }
                    if (!Int32.TryParse(txtWhereData.Text, out DataInt))
                    {
                        WhereAdd.BackColor = Color.Red;
                        return;
                    }
                }
                else if (Fields[selected] == typeof(string))
                {
                    if (thisOp.Contains("Contain") && txtWhereData.Text == "")
                    {
                        WhereAdd.BackColor = Color.Red;
                        return;
                    }
                    Data = txtWhereData.Text;
                }
            }
            var AddWhere = "";

            if (Fields.ContainsKey(selected))
            {
                if (Fields[selected] == typeof(int))
                    AddWhere = selected + " " + thisOp + DataInt;
                else if (Fields[selected] == typeof(bool))
                    AddWhere = (thisOp == "true" ? "" : "!") + selected;
                else if (Fields[selected] == typeof(string))
                {
                    switch (thisOp)
                    {
                        case "Contains":
                            AddWhere = selected + ".Contains(\"" + Data + "\")";
                            break;
                        case "Doesn't Contain":
                            AddWhere = "!" + selected + ".Contains(\"" + Data + "\")";
                            break;
                        default:
                            AddWhere = selected + " " + thisOp + " \"" + Data + "\"";
                            break;
                    }
                }
            }
            else
            {
                AddWhere = selected + " = \"" + thisOp + "\"";
            }
            if (lstWhere.Items.Contains(AddWhere) || AddWhere == "")
            {
                WhereAdd.BackColor = Color.Red;
                return;
            }
            lstWhere.Items.Add(AddWhere);
            WhereAdd.BackColor = DefaultBackColor;
        }

        private void WhereDelete_Click(object sender, EventArgs e)
        {
            if (lstWhere.SelectedItem != null)
                lstWhere.Items.Remove(lstWhere.SelectedItem);
        }

        private void OrderDelete_Click(object sender, EventArgs e)
        {
            if (lstOrder.SelectedItem != null)
                lstOrder.Items.Remove(lstOrder.SelectedItem);
        }

        private void OrderAdd_Click(object sender, EventArgs e)
        {
            if (cmbOrderField.SelectedIndex == -1)
            {
                OrderAdd.BackColor = Color.Red;
                return;
            }
            var selected = cmbOrderField.SelectedItem.ToString();
            var thisOp = cmbOrderOperation.SelectedItem == null ? null : cmbOrderOperation.SelectedItem.ToString();
            var DataInt = 0;
            var Data = "";

            if (Fields.ContainsKey(selected))
            {
                if (Fields[selected] == typeof (int))
                {
                    if (txtOrderData.Text == "" && cmbOrderOperation.SelectedIndex != -1)
                    {
                        OrderAdd.BackColor = Color.Red;
                        return;
                    }
                    if (!Int32.TryParse(txtOrderData.Text, out DataInt) && cmbOrderOperation.SelectedIndex != -1)
                    {
                        OrderAdd.BackColor = Color.Red;
                        return;
                    }
                }
                else if (Fields[selected] == typeof (string))
                {
                    if (thisOp != null && thisOp.Contains("Contain") && txtOrderData.Text == "")
                    {
                        OrderAdd.BackColor = Color.Red;
                        return;
                    }
                    Data = txtOrderData.Text;
                }
            }
            var AddOrder = "";

            if (thisOp == null)
               AddOrder = selected;
            else
            {
                if (Fields.ContainsKey(selected))
                {
                    if (Fields[selected] == typeof (int))
                        AddOrder = selected + " " + thisOp + DataInt;
                    else if (Fields[selected] == typeof (bool))
                        AddOrder = (thisOp == "true" ? "" : "!") + selected;
                    else if (Fields[selected] == typeof (string))
                    {
                        switch (thisOp)
                        {
                            case "Contains":
                                AddOrder = selected + ".Contains(\"" + Data + "\")";
                                break;
                            case "Doesn't Contain":
                                AddOrder = "!" + selected + ".Contains(\"" + Data + "\")";
                                break;
                            default:
                                AddOrder = selected + " " + thisOp + " \"" + Data + "\"";
                                break;
                        }
                    }
                }
                else
                {
                    AddOrder = selected + " = \"" + thisOp + "\"";
                }
            }
            if (AddOrder != "" && chkDescending.Checked)
                AddOrder = "-" + AddOrder;

            if (lstOrder.Items.Contains(AddOrder) || AddOrder == "")
            {
                OrderAdd.BackColor = Color.Red;
                return;
            }
            lstOrder.Items.Add(AddOrder);
            OrderAdd.BackColor = DefaultBackColor;
        }

        private void cmbOrderField_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selected = ((ComboBox)sender).SelectedItem.ToString();
            cmbOrderOperation.Items.Clear();
            if (Fields.ContainsKey(selected))
            {
                if (Fields[selected] == typeof (bool))
                {
                    cmbOrderOperation.Items.AddRange(boolOps.ToArray<object>());
                    txtOrderData.Visible = false;
                }
                else if (Fields[selected] == typeof (int))
                {
                    cmbOrderOperation.Items.AddRange(intOps.ToArray<object>());
                    txtOrderData.Visible = true;
                }
                else if (Fields[selected] == typeof (string))
                {
                    cmbOrderOperation.Items.AddRange(stringOps.ToArray<object>());
                    txtOrderData.Visible = true;
                }
            }
            else
            {
                var addRange = Options[selected].ToList();
                addRange.Sort();
                cmbOrderOperation.Items.AddRange(addRange.ToArray<object>());
                txtOrderData.Visible = false;
            }
            txtOrderData.Text = "";
        }

        private void OrderMoveUp_Click(object sender, EventArgs e)
        {
            if (lstOrder.SelectedItem == null || lstOrder.SelectedIndex == 0) return;
            var CurIndex = lstOrder.SelectedIndex;
            var curItem = lstOrder.SelectedItem.ToString();
            lstOrder.Items.Remove(lstOrder.SelectedItem);
            lstOrder.Items.Insert(CurIndex-1,curItem);
            lstOrder.SelectedItem = curItem;
        }

        private void OrderMoveDown_Click(object sender, EventArgs e)
        {
            if (lstOrder.SelectedItem == null || lstOrder.SelectedIndex == lstOrder.Items.Count - 1) return;
            var CurIndex = lstOrder.SelectedIndex;
            var curItem = lstOrder.SelectedItem.ToString();
            lstOrder.Items.Remove(lstOrder.SelectedItem);
            lstOrder.Items.Insert(CurIndex + 1, curItem);
            lstOrder.SelectedItem = curItem;
        }

        private void Apply_Click(object sender, EventArgs e)
        {
            List<string> Wheres = null;
            List<string> Orders = null;
            if (lstWhere.Items.Count > 0)
            {
                Wheres = lstWhere.Items.Cast<string>().ToList();
            }
            if (lstOrder.Items.Count > 0)
            {
                Orders = lstOrder.Items.Cast<string>().ToList();
            }
            int TakeI;
            if (!(chkTake.Checked && Int32.TryParse(txtTake.Text,out TakeI)))
                TakeI = -1;

            outFilter = new Filter(Orders, Wheres, TakeI);
        }
    }
}
